﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion



using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using HeuristicLab.Data;
using HeuristicLab.Common;
using HeuristicLab.Parameters;
using System;
using HeuristicLab.Problems.Scheduling.Encodings.JobSequenceMatrix;
using HeuristicLab.Problems.Scheduling.Encodings;
using HeuristicLab.Operators;


namespace HeuristicLab.Problems.Scheduling.Evaluators {
  [Item("JSSP Evaluator", "Represents a evaluator class for standard job shop scheduling problems.")]
  [StorableClass]
  public abstract class JSSPEvaluator : JSSPOperator, ISingleObjectiveEvaluator {
    [StorableConstructor]
    protected JSSPEvaluator(bool deserializing) : base(deserializing) { }
    protected JSSPEvaluator(JSSPEvaluator original, Cloner cloner)
      : base(original, cloner) {
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { 
        if (Parameters.ContainsKey("Quality"))
          return (ILookupParameter<DoubleValue>)Parameters["Quality"];
        else
          return null;
      }
    }
    public ILookupParameter<Schedule> DecodedSchedulingSolutionParameter {
      get { return (ILookupParameter<Schedule>)Parameters["DecodedSchedulingSolution"]; }
    }

    public JSSPEvaluator () : base () {
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The quality value aka fitness value of the solution."));
      Parameters.Add(new LookupParameter<Schedule>("DecodedSchedulingSolution", "The decoded scheduling solution represented as generalized schedule."));
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
    }

    protected abstract DoubleValue evaluate(Schedule schedule);
    
    public override IOperation Apply() {
      Schedule decodedSchedulingSolution = DecodedSchedulingSolutionParameter.ActualValue;
      QualityParameter.ActualValue = evaluate(decodedSchedulingSolution);
      return base.Apply();
    }
  }
}
